@@ -128,7 +128,7 @@ static void s_testing_channel_shutdown(int error_code, void *user_data) {
         return;
     }
     if (fake_connection->options.on_shutdown) {
-        /* In real world, this is trigger by the bootstrp */
+        /* In real world, this is trigger by the bootstrap */
         fake_connection->options.on_shutdown(
             fake_connection->connection, error_code, fake_connection->options.user_data);
     }
@@ -236,7 +236,7 @@ static int s_tester_init(struct sm_tester_options *options) {
     if (options->uri_cursor) {
         ASSERT_SUCCESS(aws_uri_init_parse(&s_tester.endpoint, alloc, options->uri_cursor));
     } else {
-        struct aws_byte_cursor default_host = aws_byte_cursor_from_c_str("https://example.com");
+        struct aws_byte_cursor default_host = aws_byte_cursor_from_c_str("https://www.amazon.com");
         ASSERT_SUCCESS(aws_uri_init_parse(&s_tester.endpoint, alloc, &default_host));
     }
 
@@ -1229,7 +1229,7 @@ TEST_CASE(h2_sm_acquire_stream_multiple_connections) {
 TEST_CASE(h2_sm_close_connection_on_server_error) {
     (void)ctx;
     /* server that will return 500 status code all the time. */
-    struct aws_byte_cursor uri_cursor = aws_byte_cursor_from_c_str("https://httpbin.org/status/500");
+    struct aws_byte_cursor uri_cursor = aws_byte_cursor_from_c_str("https://postman-echo.com/status/500");
     struct sm_tester_options options = {
         .max_connections = 1,
         .max_concurrent_streams_per_connection = 10,
@@ -1399,12 +1399,14 @@ static int s_sm_stream_acquiring_with_body(int num_streams) {
 TEST_CASE(localhost_integ_h2_sm_acquire_stream_stress_with_body) {
     (void)ctx;
     struct aws_byte_cursor uri_cursor = aws_byte_cursor_from_c_str("https://localhost:3443/upload_test");
+    enum aws_log_level log_level = AWS_LOG_LEVEL_DEBUG;
     struct sm_tester_options options = {
         .max_connections = 100,
         .max_concurrent_streams_per_connection = 100,
         .connection_ping_period_ms = 100 * AWS_TIMESTAMP_MILLIS,
         .alloc = allocator,
         .uri_cursor = &uri_cursor,
+        .log_level = &log_level,
     };
     ASSERT_SUCCESS(s_tester_init(&options));
     s_tester.length_sent = 2000;
