@@ -12,6 +12,7 @@
 #include <aws/common/mutex.h>
 #include <aws/common/ref_count.h>
 #include <aws/io/channel.h>
+#include <aws/mqtt/private/v5/mqtt5_callbacks.h>
 #include <aws/mqtt/private/v5/mqtt5_decoder.h>
 #include <aws/mqtt/private/v5/mqtt5_encoder.h>
 #include <aws/mqtt/private/v5/mqtt5_topic_alias.h>
@@ -341,7 +342,7 @@ struct aws_mqtt5_client {
     /*
      * Client configuration
      */
-    const struct aws_mqtt5_client_options_storage *config;
+    struct aws_mqtt5_client_options_storage *config;
 
     /*
      * The recurrent task that runs all client logic outside of external event callbacks.  Bound to the client's
@@ -439,6 +440,11 @@ struct aws_mqtt5_client {
      */
     struct aws_mqtt5_client_flow_control_state flow_control_state;
 
+    /*
+     * Manages notification listener chains for lifecycle events and incoming publishes
+     */
+    struct aws_mqtt5_callback_set_manager callback_manager;
+
     /*
      * When should the next PINGREQ be sent?
      */
@@ -483,6 +489,29 @@ struct aws_mqtt5_client {
      * with clean start set to false.
      */
     bool has_connected_successfully;
+
+    /*
+     * A flag that allows in-thread observers (currently the mqtt3_to_5 adapter) to signal that the connection
+     * should be torn down and re-established.  Only relevant to the CONNECTING state which is not interruptible:
+     *
+     * If the mqtt5 client is in the CONNECTING state (ie waiting for bootstrap to complete) and the 3-adapter
+     * is asked to connect, then we *MUST* discard the in-progress connection attempt in order to guarantee the
+     * connection we establish uses all of the configuration parameters that are passed during the mqtt3 API's connect
+     * call (host, port, tls options, socket options, etc...).  Since we can't interrupt the CONNECTING state, we
+     * instead set a flag that tells the mqtt5 client to tear down the connection as soon as the initial bootstrap
+     * completes.  The reconnect will establish the requested connection using the parameters passed to
+     * the mqtt3 API.
+     *
+     * Rather than try and catch every escape path from CONNECTING, we lazily reset this flag to false when we
+     * enter the CONNECTING state.  On a similar note, we only check this flag as we transition to MQTT_CONNECT.
+     *
+     * This flag is ultimately only needed when the 3 adapter and 5 client are used out-of-sync.  If you use the
+     * 3 adapter exclusively after 5 client creation, it never comes into play.
+     *
+     * Even the adapter shouldn't manipulate this directly.  Instead, use the aws_mqtt5_client_reset_connection private
+     * API to tear down an in-progress or established connection in response to a connect() request on the adapter.
+     */
+    bool should_reset_connection;
 };
 
 AWS_EXTERN_C_BEGIN
@@ -632,6 +661,18 @@ AWS_MQTT_API void aws_mqtt5_client_statistics_change_operation_statistic_state(
  */
 AWS_MQTT_API const char *aws_mqtt5_client_state_to_c_string(enum aws_mqtt5_client_state state);
 
+/**
+ * An internal API used by the MQTT3 adapter to force any existing-or-in-progress connection to
+ * be torn down and re-established.  Necessary because the MQTT3 interface allows overrides on a large number
+ * of configuration parameters through the connect() call.  We must honor those parameters and the safest thing
+ * to do is to just throw away the current connection (if it exists) and make a new one.  In the case that an MQTT5
+ * client is being driven entirely by the MQTT3 adapter, this case never actually happens.
+ *
+ * @param client client to reset an existing or in-progress connection for
+ * @return true if a connection reset was triggered, false if there was nothing to do
+ */
+AWS_MQTT_API bool aws_mqtt5_client_reset_connection(struct aws_mqtt5_client *client);
+
 AWS_EXTERN_C_END
 
 #endif /* AWS_MQTT_MQTT5_CLIENT_IMPL_H */
