@@ -6,6 +6,7 @@
 #include "aws/s3/private/s3_auto_ranged_get.h"
 #include "aws/s3/private/s3_auto_ranged_put.h"
 #include "aws/s3/private/s3_client_impl.h"
+#include "aws/s3/private/s3_copy_object.h"
 #include "aws/s3/private/s3_default_meta_request.h"
 #include "aws/s3/private/s3_meta_request_impl.h"
 #include "aws/s3/private/s3_request_messages.h"
@@ -16,7 +17,6 @@
 #include <aws/common/atomics.h>
 #include <aws/common/clock.h>
 #include <aws/common/device_random.h>
-#include <aws/common/environment.h>
 #include <aws/common/json.h>
 #include <aws/common/string.h>
 #include <aws/common/system_info.h>
@@ -33,7 +33,6 @@
 #include <aws/io/tls_channel_handler.h>
 #include <aws/io/uri.h>
 
-#include <aws/s3/private/s3_copy_object.h>
 #include <inttypes.h>
 #include <math.h>
 
@@ -937,12 +936,12 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
     AWS_PRECONDITION(client);
     AWS_PRECONDITION(options);
 
-    struct aws_http_headers *initial_message_headers = aws_http_message_get_headers(options->message);
+    const struct aws_http_headers *initial_message_headers = aws_http_message_get_headers(options->message);
     AWS_ASSERT(initial_message_headers);
 
     uint64_t content_length = 0;
     struct aws_byte_cursor content_length_cursor;
-    bool content_length_header_found = false;
+    bool content_length_found = false;
 
     if (!aws_http_headers_get(initial_message_headers, g_content_length_header_name, &content_length_cursor)) {
         if (aws_byte_cursor_utf8_parse_u64(content_length_cursor, &content_length)) {
@@ -953,7 +952,27 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
             aws_raise_error(AWS_ERROR_S3_INVALID_CONTENT_LENGTH_HEADER);
             return NULL;
         }
-        content_length_header_found = true;
+        content_length_found = true;
+    }
+
+    /* There are multiple ways to pass the body in, ensure only 1 was used */
+    int body_source_count = 0;
+    if (aws_http_message_get_body_stream(options->message) != NULL) {
+        ++body_source_count;
+    }
+    if (options->send_filepath.len > 0) {
+        ++body_source_count;
+    }
+    if (options->send_async_stream != NULL) {
+        ++body_source_count;
+    }
+    if (body_source_count > 1) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "Could not create auto-ranged-put meta request."
+            " More than one data source is set (filepath, async stream, body stream).");
+        aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        return NULL;
     }
 
     /* Call the appropriate meta-request new function. */
@@ -970,21 +989,11 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
             return aws_s3_meta_request_auto_ranged_get_new(client->allocator, client, client->part_size, options);
         }
         case AWS_S3_META_REQUEST_TYPE_PUT_OBJECT: {
-
-            if (!content_length_header_found) {
+            if (body_source_count == 0) {
                 AWS_LOGF_ERROR(
                     AWS_LS_S3_META_REQUEST,
-                    "Could not create auto-ranged-put meta request; there is no Content-Length header present.");
-                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-                return NULL;
-            }
-
-            struct aws_input_stream *input_stream = aws_http_message_get_body_stream(options->message);
-
-            if ((input_stream == NULL) && (options->send_filepath.len == 0)) {
-                AWS_LOGF_ERROR(
-                    AWS_LS_S3_META_REQUEST,
-                    "Could not create auto-ranged-put meta request; filepath or body stream must be set.");
+                    "Could not create auto-ranged-put meta request."
+                    " Body must be set via filepath, async stream, or body stream.");
                 aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
                 return NULL;
             }
@@ -1018,7 +1027,8 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
                 }
                 uint64_t multipart_upload_threshold =
                     client->multipart_upload_threshold == 0 ? client_part_size : client->multipart_upload_threshold;
-                if (content_length <= multipart_upload_threshold) {
+
+                if (content_length_found && content_length <= multipart_upload_threshold) {
                     return aws_s3_meta_request_default_new(
                         client->allocator,
                         client,
@@ -1028,72 +1038,44 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
                         options);
                 } else {
                     if (aws_s3_message_util_check_checksum_header(options->message)) {
-                        /* The checksum header has been set and the request will be splitted. We fail the request */
+                        /* The checksum header has been set and the request will be split. We fail the request */
                         AWS_LOGF_ERROR(
                             AWS_LS_S3_META_REQUEST,
                             "Could not create auto-ranged-put meta request; checksum headers has been set for "
                             "auto-ranged-put that will be split. Pre-calculated checksums are only supported for "
-                            "single "
-                            "part upload.");
+                            "single part upload.");
                         aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
                         return NULL;
                     }
                 }
 
-                uint64_t part_size_uint64 = content_length / (uint64_t)g_s3_max_num_upload_parts;
+                size_t part_size = client_part_size;
+                uint32_t num_parts = 0;
+                if (content_length_found) {
+                    if (aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
+                            content_length, client_part_size, client_max_part_size, &part_size, &num_parts)) {
+                        return NULL;
+                    }
+                }
 
-                if (part_size_uint64 > SIZE_MAX) {
+                return aws_s3_meta_request_auto_ranged_put_new(
+                    client->allocator, client, part_size, content_length_found, content_length, num_parts, options);
+            } else { /* else using resume token */
+                if (!content_length_found) {
                     AWS_LOGF_ERROR(
                         AWS_LS_S3_META_REQUEST,
-                        "Could not create auto-ranged-put meta request; required part size of %" PRIu64
-                        " bytes is too large for platform.",
-                        part_size_uint64);
-
+                        "Could not create auto-ranged-put resume meta request; content_length must be specified.");
                     aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
                     return NULL;
                 }
 
-                size_t part_size = (size_t)part_size_uint64;
-
-                if (part_size > client_max_part_size) {
-                    AWS_LOGF_ERROR(
-                        AWS_LS_S3_META_REQUEST,
-                        "Could not create auto-ranged-put meta request; required part size for put request is %" PRIu64
-                        ", but current maximum part size is %" PRIu64,
-                        (uint64_t)part_size,
-                        (uint64_t)client_max_part_size);
-                    aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-                    return NULL;
-                }
-
-                if (part_size < client_part_size) {
-                    part_size = client_part_size;
-                }
-                if (content_length < part_size) {
-                    /* When the content length is smaller than part size and larger than the threshold, we set one part
-                     * with the whole length */
-                    part_size = (size_t)content_length;
-                }
-
-                uint32_t num_parts = (uint32_t)(content_length / part_size);
-
-                if ((content_length % part_size) > 0) {
-                    ++num_parts;
-                }
-
+                /* don't pass part size and total num parts. constructor will pick it up from token */
                 return aws_s3_meta_request_auto_ranged_put_new(
-                    client->allocator, client, part_size, content_length, num_parts, options);
-            } else {
-                /* dont pass part size and total num parts. constructor will pick it up from token */
-                return aws_s3_meta_request_auto_ranged_put_new(
-                    client->allocator, client, 0, content_length, 0, options);
+                    client->allocator, client, 0, true, content_length, 0, options);
             }
         }
         case AWS_S3_META_REQUEST_TYPE_COPY_OBJECT: {
-            /* TODO: support copy object correctly. */
-            AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "CopyObject is not currently supported");
-            aws_raise_error(AWS_ERROR_UNIMPLEMENTED);
-            return NULL;
+            return aws_s3_meta_request_copy_object_new(client->allocator, client, options);
         }
         case AWS_S3_META_REQUEST_TYPE_DEFAULT:
             return aws_s3_meta_request_default_new(client->allocator, client, content_length, false, options);
@@ -1494,7 +1476,10 @@ static void s_s3_client_prepare_callback_queue_request(
     struct aws_s3_client *client = user_data;
     AWS_PRECONDITION(client);
 
-    if (error_code != AWS_ERROR_SUCCESS) {
+    bool request_is_noop = false;
+
+    if (error_code != AWS_ERROR_SUCCESS || request->is_noop) {
+        request_is_noop = request->is_noop != 0;
         s_s3_client_meta_request_finished_request(client, meta_request, request, error_code);
 
         aws_s3_request_release(request);
@@ -1506,7 +1491,9 @@ static void s_s3_client_prepare_callback_queue_request(
         aws_s3_client_lock_synced_data(client);
 
         if (error_code == AWS_ERROR_SUCCESS) {
-            aws_linked_list_push_back(&client->synced_data.prepared_requests, &request->node);
+            if (!request_is_noop) {
+                aws_linked_list_push_back(&client->synced_data.prepared_requests, &request->node);
+            }
         } else {
             ++client->synced_data.num_failed_prepare_requests;
         }
@@ -1596,9 +1583,9 @@ static void s_s3_client_create_connection_for_request_default(
     struct aws_http_headers *message_headers = aws_http_message_get_headers(meta_request->initial_request_message);
     AWS_ASSERT(message_headers);
 
-    int get_header_result = aws_http_headers_get(message_headers, g_host_header_name, &host_header_value);
-    AWS_ASSERT(get_header_result == AWS_OP_SUCCESS);
-    (void)get_header_result;
+    int result = aws_http_headers_get(message_headers, g_host_header_name, &host_header_value);
+    AWS_ASSERT(result == AWS_OP_SUCCESS);
+    (void)result;
 
     if (aws_retry_strategy_acquire_retry_token(
             client->retry_strategy, &host_header_value, s_s3_client_acquired_retry_token, connection, 0)) {
@@ -1750,8 +1737,11 @@ void aws_s3_client_notify_connection_finished(
 
     struct aws_s3_endpoint *endpoint = meta_request->endpoint;
     AWS_PRECONDITION(endpoint);
+    if (request->send_data.metrics) {
+        request->send_data.metrics->crt_info_metrics.error_code = error_code;
+    }
 
-    /* If we're trying to setup a retry... */
+    /* If we're trying to set up a retry... */
     if (finish_code == AWS_S3_CONNECTION_FINISH_CODE_RETRY) {
 
         if (connection->retry_token == NULL) {
