@@ -16,8 +16,10 @@
 #include <aws/common/encoding.h>
 #include <aws/common/string.h>
 #include <aws/common/system_info.h>
+#include <aws/io/async_stream.h>
 #include <aws/io/event_loop.h>
 #include <aws/io/retry_strategy.h>
+#include <aws/io/socket.h>
 #include <aws/io/stream.h>
 #include <inttypes.h>
 
@@ -54,6 +56,11 @@ static int s_s3_meta_request_incoming_headers(
     size_t headers_count,
     void *user_data);
 
+static void s_s3_meta_request_stream_metrics(
+    struct aws_http_stream *stream,
+    const struct aws_http_stream_metrics *metrics,
+    void *user_data);
+
 static void s_s3_meta_request_stream_complete(struct aws_http_stream *stream, int error_code, void *user_data);
 
 static void s_s3_meta_request_send_request_finish(
@@ -105,7 +112,7 @@ static int s_meta_request_get_response_headers_checksum_callback(
     }
 }
 
-/* warning this might get screwed up with retrys/restarts */
+/* warning this might get screwed up with retries/restarts */
 static int s_meta_request_get_response_body_checksum_callback(
     struct aws_s3_meta_request *meta_request,
     const struct aws_byte_cursor *body,
@@ -223,7 +230,8 @@ int aws_s3_meta_request_init_base(
         meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
     }
 
-    /* Set initial_meta_request */
+    /* Set initial_meta_request, based on how the request's body is being passed in
+     * (we checked earlier that it's not being passed multiple ways) */
     if (options->send_filepath.len > 0) {
         /* Create copy of original message, but with body-stream that reads directly from file */
         meta_request->initial_request_message = aws_s3_message_util_copy_http_message_filepath_body_all_headers(
@@ -231,8 +239,14 @@ int aws_s3_meta_request_init_base(
         if (meta_request->initial_request_message == NULL) {
             goto error;
         }
+
+    } else if (options->send_async_stream != NULL) {
+        /* Read from async body-stream, but keep original message around for headers, method, etc */
+        meta_request->request_body_async_stream = aws_async_input_stream_acquire(options->send_async_stream);
+        meta_request->initial_request_message = aws_http_message_acquire(options->message);
+
     } else {
-        /* Keep a reference to the original message structure passed in. */
+        /* Keep original message around, we'll read from its synchronous body-stream */
         meta_request->initial_request_message = aws_http_message_acquire(options->message);
     }
 
@@ -250,6 +264,7 @@ int aws_s3_meta_request_init_base(
     meta_request->user_data = options->user_data;
     meta_request->shutdown_callback = options->shutdown_callback;
     meta_request->progress_callback = options->progress_callback;
+    meta_request->telemetry_callback = options->telemetry_callback;
 
     if (meta_request->checksum_config.validate_response_checksum) {
         /* TODO: the validate for auto range get should happen for each response received. */
@@ -418,10 +433,8 @@ static void s_s3_meta_request_destroy(void *user_data) {
     AWS_LOGF_DEBUG(AWS_LS_S3_META_REQUEST, "id=%p Cleaning up meta request", (void *)meta_request);
 
     /* Clean up our initial http message */
-    if (meta_request->initial_request_message != NULL) {
-        aws_http_message_release(meta_request->initial_request_message);
-        meta_request->initial_request_message = NULL;
-    }
+    meta_request->request_body_async_stream = aws_async_input_stream_release(meta_request->request_body_async_stream);
+    meta_request->initial_request_message = aws_http_message_release(meta_request->initial_request_message);
 
     void *meta_request_user_data = meta_request->user_data;
     aws_s3_meta_request_shutdown_fn *shutdown_callback = meta_request->shutdown_callback;
@@ -452,11 +465,11 @@ static void s_s3_meta_request_destroy(void *user_data) {
 }
 
 static int s_s3_request_priority_queue_pred(const void *a, const void *b) {
-    const struct aws_s3_request **request_a = (const struct aws_s3_request **)a;
+    const struct aws_s3_request *const *request_a = a;
     AWS_PRECONDITION(request_a);
     AWS_PRECONDITION(*request_a);
 
-    const struct aws_s3_request **request_b = (const struct aws_s3_request **)b;
+    const struct aws_s3_request *const *request_b = b;
     AWS_PRECONDITION(request_b);
     AWS_PRECONDITION(*request_b);
 
@@ -499,7 +512,10 @@ bool aws_s3_meta_request_is_finished(struct aws_s3_meta_request *meta_request) {
 }
 
 static void s_s3_meta_request_prepare_request_task(struct aws_task *task, void *arg, enum aws_task_status task_status);
+static void s_s3_meta_request_on_request_prepared(void *user_data);
 
+/* TODO: document how this is final step in prepare-request sequence.
+ * Could be invoked on any thread. */
 static void s_s3_prepare_request_payload_callback_and_destroy(
     struct aws_s3_prepare_request_payload *payload,
     int error_code) {
@@ -509,18 +525,30 @@ static void s_s3_prepare_request_payload_callback_and_destroy(
     struct aws_s3_meta_request *meta_request = payload->request->meta_request;
     AWS_PRECONDITION(meta_request);
 
-    AWS_PRECONDITION(meta_request->client);
-    struct aws_s3_client *client = aws_s3_client_acquire(meta_request->client);
+    ++payload->request->num_times_prepared;
 
-    struct aws_allocator *allocator = client->allocator;
-    AWS_PRECONDITION(allocator);
+    if (error_code) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p Could not prepare request %p due to error %d (%s).",
+            (void *)meta_request,
+            (void *)payload->request,
+            error_code,
+            aws_error_str(error_code));
+
+        /* BEGIN CRITICAL SECTION */
+        aws_s3_meta_request_lock_synced_data(meta_request);
+        aws_s3_meta_request_set_fail_synced(meta_request, payload->request, error_code);
+        aws_s3_meta_request_unlock_synced_data(meta_request);
+        /* END CRITICAL SECTION */
+    }
 
     if (payload->callback != NULL) {
         payload->callback(meta_request, payload->request, error_code, payload->user_data);
     }
 
-    aws_mem_release(allocator, payload);
-    aws_s3_client_release(client);
+    aws_future_void_release(payload->asyncstep_prepare_request);
+    aws_mem_release(payload->allocator, payload);
 }
 
 static void s_s3_meta_request_schedule_prepare_request_default(
@@ -561,6 +589,7 @@ static void s_s3_meta_request_schedule_prepare_request_default(
     struct aws_s3_prepare_request_payload *payload =
         aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_prepare_request_payload));
 
+    payload->allocator = allocator;
     payload->request = request;
     payload->callback = callback;
     payload->user_data = user_data;
@@ -583,54 +612,41 @@ static void s_s3_meta_request_prepare_request_task(struct aws_task *task, void *
     struct aws_s3_meta_request *meta_request = request->meta_request;
     AWS_PRECONDITION(meta_request);
 
-    struct aws_s3_meta_request_vtable *vtable = meta_request->vtable;
+    const struct aws_s3_meta_request_vtable *vtable = meta_request->vtable;
     AWS_PRECONDITION(vtable);
 
     /* Client owns this event loop group. A cancel should not be possible. */
     AWS_ASSERT(task_status == AWS_TASK_STATUS_RUN_READY);
 
-    int error_code = AWS_ERROR_SUCCESS;
-
     if (!request->always_send && aws_s3_meta_request_has_finish_result(meta_request)) {
-        aws_raise_error(AWS_ERROR_S3_CANCELED);
-        goto dont_send_clean_up;
+        s_s3_prepare_request_payload_callback_and_destroy(payload, AWS_ERROR_S3_CANCELED);
+        return;
     }
 
-    if (vtable->prepare_request(meta_request, request)) {
-        ++request->num_times_prepared;
-        goto dont_send_clean_up;
-    }
+    /* Kick off the async vtable->prepare_request()
+     * Each subclass has its own implementation of this. */
+    payload->asyncstep_prepare_request = vtable->prepare_request(request);
+    aws_future_void_register_callback(
+        payload->asyncstep_prepare_request, s_s3_meta_request_on_request_prepared, payload);
+    return;
+}
 
-    ++request->num_times_prepared;
+/* Called after vtable->prepare_request has succeeded or failed. */
+static void s_s3_meta_request_on_request_prepared(void *user_data) {
+    struct aws_s3_prepare_request_payload *payload = user_data;
+    struct aws_s3_request *request = payload->request;
+    struct aws_s3_meta_request *meta_request = request->meta_request;
+
+    int error_code = aws_future_void_get_error(payload->asyncstep_prepare_request);
+    if (error_code) {
+        s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
+        return;
+    }
 
     aws_s3_add_user_agent_header(meta_request->allocator, request->send_data.message);
 
-    /* Sign the newly created message. */
+    /* Next step is to sign the newly created message (completion callback could happen on any thread) */
     s_s3_meta_request_sign_request(meta_request, request, s_s3_meta_request_request_on_signed, payload);
-
-    return;
-
-dont_send_clean_up:
-
-    error_code = aws_last_error_or_unknown();
-
-    AWS_LOGF_ERROR(
-        AWS_LS_S3_META_REQUEST,
-        "id=%p Could not prepare request %p due to error %d (%s).",
-        (void *)meta_request,
-        (void *)request,
-        error_code,
-        aws_error_str(error_code));
-
-    /* BEGIN CRITICAL SECTION */
-    {
-        aws_s3_meta_request_lock_synced_data(meta_request);
-        aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
-        aws_s3_meta_request_unlock_synced_data(meta_request);
-    }
-    /* END CRITICAL SECTION */
-
-    s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
 }
 
 static void s_s3_meta_request_init_signing_date_time(
@@ -747,7 +763,7 @@ void aws_s3_meta_request_sign_request_default(
     }
 }
 
-/* Handle the signing result, getting an HTTP connection for the request if signing succeeded. */
+/* Handle the signing result */
 static void s_s3_meta_request_request_on_signed(
     struct aws_signing_result *signing_result,
     int error_code,
@@ -780,18 +796,10 @@ finish:
 
         AWS_LOGF_ERROR(
             AWS_LS_S3_META_REQUEST,
-            "id=%p Meta request could not sign TTP request due to error code %d (%s)",
+            "id=%p Meta request could not sign HTTP request due to error code %d (%s)",
             (void *)meta_request,
             error_code,
             aws_error_str(error_code));
-
-        /* BEGIN CRITICAL SECTION */
-        {
-            aws_s3_meta_request_lock_synced_data(meta_request);
-            aws_s3_meta_request_set_fail_synced(meta_request, request, error_code);
-            aws_s3_meta_request_unlock_synced_data(meta_request);
-        }
-        /* END CRITICAL SECTION */
     }
 
     s_s3_prepare_request_payload_callback_and_destroy(payload, error_code);
@@ -815,6 +823,9 @@ void aws_s3_meta_request_send_request(struct aws_s3_meta_request *meta_request,
     options.on_response_headers = s_s3_meta_request_incoming_headers;
     options.on_response_header_block_done = NULL;
     options.on_response_body = s_s3_meta_request_incoming_body;
+    if (request->send_data.metrics) {
+        options.on_metrics = s_s3_meta_request_stream_metrics;
+    }
     options.on_complete = s_s3_meta_request_stream_complete;
 
     struct aws_http_stream *stream = aws_http_connection_make_request(connection->http_connection, &options);
@@ -907,7 +918,7 @@ static void s_get_part_response_headers_checksum_helper(
     }
 }
 
-/* warning this might get screwed up with retrys/restarts */
+/* warning this might get screwed up with retries/restarts */
 static void s_get_part_response_body_checksum_helper(
     struct aws_s3_checksum *running_response_sum,
     const struct aws_byte_cursor *body) {
@@ -974,6 +985,25 @@ static int s_s3_meta_request_incoming_headers(
             (void *)meta_request,
             (void *)request);
     }
+    if (request->send_data.metrics) {
+        /* Record the headers to the metrics */
+        struct aws_s3_request_metrics *s3_metrics = request->send_data.metrics;
+        if (s3_metrics->req_resp_info_metrics.response_headers == NULL) {
+            s3_metrics->req_resp_info_metrics.response_headers = aws_http_headers_new(meta_request->allocator);
+        }
+
+        for (size_t i = 0; i < headers_count; ++i) {
+            const struct aws_byte_cursor *name = &headers[i].name;
+            const struct aws_byte_cursor *value = &headers[i].value;
+            if (aws_byte_cursor_eq(name, &g_request_id_header_name)) {
+                s3_metrics->req_resp_info_metrics.request_id =
+                    aws_string_new_from_cursor(connection->request->allocator, value);
+            }
+
+            aws_http_headers_add(s3_metrics->req_resp_info_metrics.response_headers, *name, *value);
+        }
+        s3_metrics->req_resp_info_metrics.response_status = request->send_data.response_status;
+    }
 
     bool successful_response =
         s_s3_meta_request_error_code_from_response_status(request->send_data.response_status) == AWS_ERROR_SUCCESS;
@@ -1060,6 +1090,38 @@ static int s_s3_meta_request_incoming_body(
     return AWS_OP_SUCCESS;
 }
 
+static void s_s3_meta_request_stream_metrics(
+    struct aws_http_stream *stream,
+    const struct aws_http_stream_metrics *http_metrics,
+    void *user_data) {
+    (void)stream;
+    struct aws_s3_connection *connection = user_data;
+    AWS_PRECONDITION(connection);
+
+    struct aws_s3_request *request = connection->request;
+    AWS_PRECONDITION(request);
+    AWS_ASSERT(request->send_data.metrics);
+    struct aws_s3_request_metrics *s3_metrics = request->send_data.metrics;
+    /* Copy over the time metrics from aws_http_stream_metrics to aws_s3_request_metrics */
+    s3_metrics->time_metrics.send_start_timestamp_ns = http_metrics->send_start_timestamp_ns;
+    s3_metrics->time_metrics.send_end_timestamp_ns = http_metrics->send_end_timestamp_ns;
+    s3_metrics->time_metrics.sending_duration_ns = http_metrics->sending_duration_ns;
+    s3_metrics->time_metrics.receive_start_timestamp_ns = http_metrics->receive_start_timestamp_ns;
+    s3_metrics->time_metrics.receive_end_timestamp_ns = http_metrics->receive_end_timestamp_ns;
+    s3_metrics->time_metrics.receiving_duration_ns = http_metrics->receiving_duration_ns;
+
+    s3_metrics->crt_info_metrics.stream_id = http_metrics->stream_id;
+
+    /* Also related metrics from the request/response. */
+    s3_metrics->crt_info_metrics.connection_id = (void *)connection->http_connection;
+    const struct aws_socket_endpoint *endpoint = aws_http_connection_get_remote_endpoint(connection->http_connection);
+    request->send_data.metrics->crt_info_metrics.ip_address =
+        aws_string_new_from_c_str(request->allocator, endpoint->address);
+    AWS_ASSERT(request->send_data.metrics->crt_info_metrics.ip_address != NULL);
+
+    s3_metrics->crt_info_metrics.thread_id = aws_thread_current_thread_id();
+}
+
 /* Finish up the processing of the request work. */
 static void s_s3_meta_request_stream_complete(struct aws_http_stream *stream, int error_code, void *user_data) {
 
@@ -1512,10 +1574,8 @@ void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request
     /* As the meta request has been finished with any HTTP message, we can safely release the http message that hold. So
      * that, the downstream high level language doesn't need to wait for shutdown to clean related resource (eg: input
      * stream) */
-    if (meta_request->initial_request_message) {
-        aws_http_message_release(meta_request->initial_request_message);
-        meta_request->initial_request_message = NULL;
-    }
+    meta_request->request_body_async_stream = aws_async_input_stream_release(meta_request->request_body_async_stream);
+    meta_request->initial_request_message = aws_http_message_release(meta_request->initial_request_message);
 
     if (meta_request->finish_callback != NULL) {
         meta_request->finish_callback(meta_request, &finish_result, meta_request->user_data);
@@ -1529,21 +1589,60 @@ void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request
     meta_request->io_event_loop = NULL;
 }
 
-int aws_s3_meta_request_read_body(struct aws_s3_meta_request *meta_request, struct aws_byte_buf *buffer) {
+struct aws_future_bool *aws_s3_meta_request_read_body(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_byte_buf *buffer) {
+
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(buffer);
 
+    /* If async-stream, simply call read_to_fill() */
+    if (meta_request->request_body_async_stream != NULL) {
+        return aws_async_input_stream_read_to_fill(meta_request->request_body_async_stream, buffer);
+    }
+
+    /* Else synchronous aws_input_stream */
+    struct aws_input_stream *synchronous_stream =
+        aws_http_message_get_body_stream(meta_request->initial_request_message);
+    AWS_FATAL_ASSERT(synchronous_stream);
+
+    struct aws_future_bool *synchronous_read_future = aws_future_bool_new(meta_request->allocator);
+
+    /* Keep calling read() until we fill the buffer, or hit EOF */
+    struct aws_stream_status status = {.is_end_of_stream = false, .is_valid = true};
+    while ((buffer->len < buffer->capacity) && !status.is_end_of_stream) {
+        /* Read from stream */
+        if (aws_input_stream_read(synchronous_stream, buffer) != AWS_OP_SUCCESS) {
+            aws_future_bool_set_error(synchronous_read_future, aws_last_error());
+            goto synchronous_read_done;
+        }
+
+        /* Check if stream is done */
+        if (aws_input_stream_get_status(synchronous_stream, &status) != AWS_OP_SUCCESS) {
+            aws_future_bool_set_error(synchronous_read_future, aws_last_error());
+            goto synchronous_read_done;
+        }
+    }
+
+    aws_future_bool_set_result(synchronous_read_future, status.is_end_of_stream);
+
+synchronous_read_done:
+    return synchronous_read_future;
+}
+
+bool aws_s3_meta_request_body_has_no_more_data(const struct aws_s3_meta_request *meta_request) {
+    AWS_PRECONDITION(meta_request);
+
     struct aws_input_stream *initial_body_stream =
         aws_http_message_get_body_stream(meta_request->initial_request_message);
     AWS_FATAL_ASSERT(initial_body_stream);
 
-    /* Copy it into our buffer. */
-    if (aws_input_stream_read(initial_body_stream, buffer)) {
-        AWS_LOGF_ERROR(AWS_LS_S3_META_REQUEST, "id=%p Could not read from body stream.", (void *)meta_request);
-        return AWS_OP_ERR;
+    struct aws_stream_status status;
+    if (aws_input_stream_get_status(initial_body_stream, &status)) {
+        return true;
     }
 
-    return AWS_OP_SUCCESS;
+    return status.is_end_of_stream;
 }
 
 void aws_s3_meta_request_result_setup(
