@@ -9,7 +9,6 @@
 #include <aws/common/string.h>
 #include <aws/common/xml_parser.h>
 #include <aws/http/request_response.h>
-#include <aws/s3/s3.h>
 #include <aws/s3/s3_client.h>
 #include <inttypes.h>
 
@@ -23,6 +22,7 @@ const struct aws_byte_cursor g_s3_service_name = AWS_BYTE_CUR_INIT_FROM_STRING_L
 const struct aws_byte_cursor g_host_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Host");
 const struct aws_byte_cursor g_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Range");
 const struct aws_byte_cursor g_if_match_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("If-Match");
+const struct aws_byte_cursor g_request_id_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-request-id");
 const struct aws_byte_cursor g_etag_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ETag");
 const struct aws_byte_cursor g_content_range_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Range");
 const struct aws_byte_cursor g_content_type_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Type");
@@ -288,29 +288,27 @@ void aws_s3_init_default_signing_config(
     signing_config->signed_body_value = g_aws_signed_body_value_unsigned_payload;
 }
 
-void replace_quote_entities(struct aws_allocator *allocator, struct aws_string *str, struct aws_byte_buf *out_buf) {
+static struct aws_byte_cursor s_quote_entity_literal = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("&quot;");
+static struct aws_byte_cursor s_quote_literal = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("\"");
+
+void aws_replace_quote_entities(struct aws_allocator *allocator, struct aws_string *str, struct aws_byte_buf *out_buf) {
     AWS_PRECONDITION(str);
 
     aws_byte_buf_init(out_buf, allocator, str->len);
 
-    struct aws_byte_cursor quote_entity = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("&quot;");
-    struct aws_byte_cursor quote = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("\"");
-
-    size_t i = 0;
-
-    while (i < str->len) {
+    for (size_t i = 0; i < str->len; ++i) {
         size_t chars_remaining = str->len - i;
 
-        if (chars_remaining >= quote_entity.len &&
-            !strncmp((const char *)&str->bytes[i], (const char *)quote_entity.ptr, quote_entity.len)) {
+        if (chars_remaining >= s_quote_entity_literal.len &&
+            !strncmp(
+                (const char *)&str->bytes[i], (const char *)s_quote_entity_literal.ptr, s_quote_entity_literal.len)) {
             /* Append quote */
-            aws_byte_buf_append(out_buf, &quote);
-            i += quote_entity.len;
+            aws_byte_buf_append(out_buf, &s_quote_literal);
+            i += s_quote_entity_literal.len - 1;
         } else {
             /* Append character */
             struct aws_byte_cursor character_cursor = aws_byte_cursor_from_array(&str->bytes[i], 1);
             aws_byte_buf_append(out_buf, &character_cursor);
-            ++i;
         }
     }
 }
@@ -339,7 +337,7 @@ void aws_s3_add_user_agent_header(struct aws_allocator *allocator, struct aws_ht
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(message);
 
-    const struct aws_byte_cursor space_delimeter = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(" ");
+    const struct aws_byte_cursor space_delimiter = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(" ");
     const struct aws_byte_cursor forward_slash = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/");
 
     const size_t user_agent_product_version_length =
@@ -355,16 +353,16 @@ void aws_s3_add_user_agent_header(struct aws_allocator *allocator, struct aws_ht
     AWS_ZERO_STRUCT(user_agent_buffer);
 
     if (aws_http_headers_get(headers, g_user_agent_header_name, &current_user_agent_header) == AWS_OP_SUCCESS) {
-        /* If the header was found, then create a buffer with the total size we'll need, and append the curent user
+        /* If the header was found, then create a buffer with the total size we'll need, and append the current user
          * agent header with a trailing space. */
         aws_byte_buf_init(
             &user_agent_buffer,
             allocator,
-            current_user_agent_header.len + space_delimeter.len + user_agent_product_version_length);
+            current_user_agent_header.len + space_delimiter.len + user_agent_product_version_length);
 
         aws_byte_buf_append_dynamic(&user_agent_buffer, &current_user_agent_header);
 
-        aws_byte_buf_append_dynamic(&user_agent_buffer, &space_delimeter);
+        aws_byte_buf_append_dynamic(&user_agent_buffer, &space_delimiter);
 
     } else {
         AWS_ASSERT(aws_last_error() == AWS_ERROR_HTTP_HEADER_NOT_FOUND);
@@ -546,6 +544,64 @@ void aws_s3_get_part_range(
     }
 }
 
+int aws_s3_calculate_optimal_mpu_part_size_and_num_parts(
+    uint64_t content_length,
+    size_t client_part_size,
+    uint64_t client_max_part_size,
+    size_t *out_part_size,
+    uint32_t *out_num_parts) {
+
+    AWS_FATAL_ASSERT(out_part_size);
+    AWS_FATAL_ASSERT(out_num_parts);
+
+    uint64_t part_size_uint64 = content_length / (uint64_t)g_s3_max_num_upload_parts;
+
+    if ((content_length % g_s3_max_num_upload_parts) > 0) {
+        ++part_size_uint64;
+    }
+
+    if (part_size_uint64 > SIZE_MAX) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "Could not create meta request; required part size of %" PRIu64 " bytes is too large for platform.",
+            part_size_uint64);
+
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    size_t part_size = (size_t)part_size_uint64;
+
+    if (part_size > client_max_part_size) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_META_REQUEST,
+            "Could not create meta request; required part size for request is %" PRIu64
+            ", but current maximum part size is %" PRIu64,
+            (uint64_t)part_size,
+            (uint64_t)client_max_part_size);
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    if (part_size < client_part_size) {
+        part_size = client_part_size;
+    }
+
+    if (content_length < part_size) {
+        /* When the content length is smaller than part size and larger than the threshold, we set one part
+         * with the whole length */
+        part_size = (size_t)content_length;
+    }
+
+    uint32_t num_parts = (uint32_t)(content_length / part_size);
+    if ((content_length % part_size) > 0) {
+        ++num_parts;
+    }
+    AWS_ASSERT(num_parts <= g_s3_max_num_upload_parts);
+
+    *out_part_size = part_size;
+    *out_num_parts = num_parts;
+    return AWS_OP_SUCCESS;
+}
+
 int aws_s3_crt_error_code_from_server_error_code_string(const struct aws_string *error_code_string) {
     if (aws_string_eq_byte_cursor(error_code_string, &g_s3_slow_down_error_code)) {
         return AWS_ERROR_S3_SLOW_DOWN;
