@@ -65,8 +65,8 @@ static bool s_s3_meta_request_update_cancel_test(
             block_update = !call_cancel && auto_ranged_put->synced_data.num_parts_sent == 1;
             break;
         case S3_UPDATE_CANCEL_TYPE_MPU_ALL_PARTS_COMPLETED:
-            call_cancel =
-                auto_ranged_put->synced_data.num_parts_completed == auto_ranged_put->synced_data.total_num_parts;
+            call_cancel = auto_ranged_put->synced_data.num_parts_completed ==
+                          auto_ranged_put->total_num_parts_from_content_length;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_NUM_MPU_CANCEL_TYPES:
@@ -395,12 +395,16 @@ AWS_TEST_CASE(test_s3_cancel_mpu_one_part_completed_fc, s_test_s3_cancel_mpu_one
 static int s_test_s3_cancel_mpu_one_part_completed_fc(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
-    ASSERT_SUCCESS(s3_cancel_test_helper_fc(
-        allocator,
-        S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED,
-        aws_byte_cursor_from_c_str("/prefix/cancel/upload_one_part_complete_fc.txt"),
-        AWS_SCA_CRC32));
+    struct aws_byte_buf path_buf;
+    AWS_ZERO_STRUCT(path_buf);
 
+    ASSERT_SUCCESS(aws_s3_tester_upload_file_path_init(
+        allocator, &path_buf, aws_byte_cursor_from_c_str("/prefix/cancel/upload_one_part_complete_fc.txt")));
+
+    ASSERT_SUCCESS(s3_cancel_test_helper_fc(
+        allocator, S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED, aws_byte_cursor_from_buf(&path_buf), AWS_SCA_CRC32));
+
+    aws_byte_buf_clean_up(&path_buf);
     return 0;
 }
 
@@ -409,10 +413,7 @@ static int s_test_s3_cancel_mpd_one_part_completed_fc(struct aws_allocator *allo
     (void)ctx;
 
     ASSERT_SUCCESS(s3_cancel_test_helper_fc(
-        allocator,
-        S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_COMPLETED,
-        aws_byte_cursor_from_c_str("/prefix/cancel/get_one_part_complete_fc.txt"),
-        AWS_SCA_CRC32));
+        allocator, S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_COMPLETED, g_pre_existing_object_10MB, AWS_SCA_CRC32));
 
     return 0;
 }
@@ -529,10 +530,18 @@ struct test_s3_cancel_prepare_user_data {
     uint32_t request_prepare_counters[AWS_S3_AUTO_RANGED_PUT_REQUEST_TAG_MAX];
 };
 
-static int s_test_s3_cancel_prepare_meta_request_prepare_request(
-    struct aws_s3_meta_request *meta_request,
-    struct aws_s3_request *request) {
+/* Data for async cancel-prepare-meta-request job */
+struct test_s3_cancel_prepare_meta_request_prepare_request_job {
+    struct aws_allocator *allocator;
+    struct aws_s3_request *request;
+    struct aws_future_void *original_future; /* original future that we're intercepting and patching */
+    struct aws_future_void *patched_future;  /* patched future to set when this job completes */
+};
 
+static void s_test_s3_cancel_prepare_meta_request_prepare_request_on_original_done(void *user_data);
+
+static struct aws_future_void *s_test_s3_cancel_prepare_meta_request_prepare_request(struct aws_s3_request *request) {
+    struct aws_s3_meta_request *meta_request = request->meta_request;
     AWS_ASSERT(meta_request != NULL);
 
     struct aws_s3_meta_request_test_results *results = meta_request->user_data;
@@ -541,14 +550,41 @@ static int s_test_s3_cancel_prepare_meta_request_prepare_request(
     struct aws_s3_tester *tester = results->tester;
     AWS_ASSERT(tester != NULL);
 
+    struct aws_future_void *patched_future = aws_future_void_new(meta_request->allocator);
+
+    struct test_s3_cancel_prepare_meta_request_prepare_request_job *patched_prep = aws_mem_calloc(
+        meta_request->allocator, 1, sizeof(struct test_s3_cancel_prepare_meta_request_prepare_request_job));
+    patched_prep->allocator = meta_request->allocator;
+    patched_prep->request = request;
+    patched_prep->patched_future = aws_future_void_acquire(patched_future);
+
     struct aws_s3_meta_request_vtable *original_meta_request_vtable =
         aws_s3_tester_get_meta_request_vtable_patch(tester, 0)->original_vtable;
 
-    if (original_meta_request_vtable->prepare_request(meta_request, request)) {
-        return AWS_OP_ERR;
+    patched_prep->original_future = original_meta_request_vtable->prepare_request(request);
+    aws_future_void_register_callback(
+        patched_prep->original_future,
+        s_test_s3_cancel_prepare_meta_request_prepare_request_on_original_done,
+        patched_prep);
+
+    return patched_future;
+}
+
+static void s_test_s3_cancel_prepare_meta_request_prepare_request_on_original_done(void *user_data) {
+
+    struct test_s3_cancel_prepare_meta_request_prepare_request_job *patched_prep = user_data;
+    struct aws_s3_request *request = patched_prep->request;
+    struct aws_s3_meta_request *meta_request = request->meta_request;
+    struct aws_s3_meta_request_test_results *results = meta_request->user_data;
+    struct aws_s3_tester *tester = results->tester;
+    struct test_s3_cancel_prepare_user_data *test_user_data = tester->user_data;
+
+    int error_code = aws_future_void_get_error(patched_prep->original_future);
+    if (error_code != AWS_ERROR_SUCCESS) {
+        aws_future_void_set_error(patched_prep->patched_future, error_code);
+        goto finish;
     }
 
-    struct test_s3_cancel_prepare_user_data *test_user_data = tester->user_data;
     ++test_user_data->request_prepare_counters[request->request_tag];
 
     /* Cancel after the first part is prepared, preventing any additional parts from being prepared. */
@@ -557,7 +593,11 @@ static int s_test_s3_cancel_prepare_meta_request_prepare_request(
         aws_s3_meta_request_cancel(meta_request);
     }
 
-    return AWS_OP_SUCCESS;
+    aws_future_void_set_result(patched_prep->patched_future);
+finish:
+    aws_future_void_release(patched_prep->original_future);
+    aws_future_void_release(patched_prep->patched_future);
+    aws_mem_release(patched_prep->allocator, patched_prep);
 }
 
 static struct aws_s3_meta_request *s_test_s3_cancel_prepare_meta_request_factory(
