@@ -20,7 +20,9 @@
 #include <aws/io/host_resolver.h>
 #include <aws/io/stream.h>
 #include <aws/io/tls_channel_handler.h>
+#include <aws/testing/async_stream_tester.h>
 #include <aws/testing/aws_test_harness.h>
+#include <aws/testing/stream_tester.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <time.h>
@@ -186,6 +188,69 @@ static void s_s3_test_meta_request_shutdown(void *user_data) {
 
     aws_s3_tester_notify_meta_request_shutdown(tester);
 }
+static void s_s3_test_meta_request_telemetry(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request_metrics *metrics,
+    void *user_data) {
+    (void)meta_request;
+    struct aws_s3_meta_request_test_results *meta_request_test_results = user_data;
+    struct aws_s3_tester *tester = meta_request_test_results->tester;
+    uint64_t time_stamp = 0;
+    aws_s3_request_metrics_get_start_timestamp_ns(metrics, &time_stamp);
+    AWS_FATAL_ASSERT(time_stamp > 0);
+    aws_s3_request_metrics_get_end_timestamp_ns(metrics, &time_stamp);
+    AWS_FATAL_ASSERT(time_stamp > 0);
+    aws_s3_request_metrics_get_total_duration_ns(metrics, &time_stamp);
+    AWS_FATAL_ASSERT(time_stamp > 0);
+    if (!aws_s3_request_metrics_get_send_end_timestamp_ns(metrics, &time_stamp)) {
+        AWS_FATAL_ASSERT(time_stamp > 0);
+        uint64_t start_time = 0;
+        uint64_t end_time = 0;
+        uint64_t during_time = 0;
+        int error = 0;
+        error |= aws_s3_request_metrics_get_send_start_timestamp_ns(metrics, &start_time);
+        error |= aws_s3_request_metrics_get_send_end_timestamp_ns(metrics, &end_time);
+        error |= aws_s3_request_metrics_get_sending_duration_ns(metrics, &during_time);
+        AWS_FATAL_ASSERT(error == AWS_OP_SUCCESS);
+        AWS_FATAL_ASSERT(during_time == (end_time - start_time));
+    }
+    if (!aws_s3_request_metrics_get_receive_end_timestamp_ns(metrics, &time_stamp)) {
+        AWS_FATAL_ASSERT(time_stamp > 0);
+        uint64_t start_time = 0;
+        uint64_t end_time = 0;
+        uint64_t during_time = 0;
+        int error = 0;
+        error |= aws_s3_request_metrics_get_receive_start_timestamp_ns(metrics, &start_time);
+        error |= aws_s3_request_metrics_get_receive_end_timestamp_ns(metrics, &end_time);
+        error |= aws_s3_request_metrics_get_receiving_duration_ns(metrics, &during_time);
+        AWS_FATAL_ASSERT(error == AWS_OP_SUCCESS);
+        AWS_FATAL_ASSERT(during_time == (end_time - start_time));
+    }
+
+    aws_s3_tester_lock_synced_data(tester);
+    aws_array_list_push_back(&meta_request_test_results->synced_data.metrics, &metrics);
+    aws_s3_request_metrics_acquire(metrics);
+    aws_s3_tester_unlock_synced_data(tester);
+}
+
+static void s_s3_test_meta_request_progress(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_s3_meta_request_progress *progress,
+    void *user_data) {
+
+    (void)meta_request;
+    AWS_ASSERT(meta_request);
+    AWS_ASSERT(progress);
+    AWS_ASSERT(user_data);
+
+    struct aws_s3_meta_request_test_results *meta_request_test_results = user_data;
+
+    aws_atomic_fetch_add(&meta_request_test_results->total_bytes_uploaded, (size_t)progress->bytes_transferred);
+
+    if (meta_request_test_results->progress_callback != NULL) {
+        meta_request_test_results->progress_callback(meta_request, progress, user_data);
+    }
+}
 
 /* Notify the tester that a particular clean up step has finished. */
 static void s_s3_test_client_shutdown(void *user_data);
@@ -353,7 +418,11 @@ int aws_s3_tester_bind_meta_request(
     ASSERT_TRUE(options->shutdown_callback == NULL);
     options->shutdown_callback = s_s3_test_meta_request_shutdown;
 
-    options->progress_callback = meta_request_test_results->progress_callback;
+    ASSERT_TRUE(options->telemetry_callback == NULL);
+    options->telemetry_callback = s_s3_test_meta_request_telemetry;
+
+    ASSERT_TRUE(options->progress_callback == NULL);
+    options->progress_callback = s_s3_test_meta_request_progress;
 
     ASSERT_TRUE(options->user_data == NULL);
     options->user_data = meta_request_test_results;
@@ -369,6 +438,8 @@ void aws_s3_meta_request_test_results_init(
     AWS_ZERO_STRUCT(*test_meta_request);
     aws_atomic_init_int(&test_meta_request->received_body_size_delta, 0);
     aws_atomic_init_int(&test_meta_request->total_bytes_uploaded, 0);
+    aws_array_list_init_dynamic(
+        &test_meta_request->synced_data.metrics, allocator, 4, sizeof(struct aws_s3_request_metrics *));
 }
 
 void aws_s3_meta_request_test_results_clean_up(struct aws_s3_meta_request_test_results *test_meta_request) {
@@ -379,6 +450,13 @@ void aws_s3_meta_request_test_results_clean_up(struct aws_s3_meta_request_test_r
     aws_http_headers_release(test_meta_request->error_response_headers);
     aws_byte_buf_clean_up(&test_meta_request->error_response_body);
     aws_http_headers_release(test_meta_request->response_headers);
+    while (aws_array_list_length(&test_meta_request->synced_data.metrics) > 0) {
+        struct aws_s3_request_metrics *metrics = NULL;
+        aws_array_list_back(&test_meta_request->synced_data.metrics, (void **)&metrics);
+        aws_array_list_pop_back(&test_meta_request->synced_data.metrics);
+        aws_s3_request_metrics_release(metrics);
+    }
+    aws_array_list_clean_up(&test_meta_request->synced_data.metrics);
 
     AWS_ZERO_STRUCT(*test_meta_request);
 }
@@ -737,12 +815,10 @@ static void s_s3_meta_request_schedule_prepare_request_empty(
     (void)user_data;
 }
 
-static int s_s3_meta_request_prepare_request_empty(
-    struct aws_s3_meta_request *meta_request,
-    struct aws_s3_request *request) {
-    (void)meta_request;
-    (void)request;
-    return AWS_OP_ERR;
+static struct aws_future_void *s_s3_meta_request_prepare_request_async_empty(struct aws_s3_request *request) {
+    struct aws_future_void *future = aws_future_void_new(request->allocator);
+    aws_future_void_set_error(future, AWS_ERROR_UNKNOWN);
+    return future;
 }
 
 static void s_s3_meta_request_init_signing_date_time_empty(
@@ -773,7 +849,7 @@ static struct aws_s3_meta_request_vtable s_s3_mock_meta_request_vtable = {
     .update = s_s3_meta_request_update_empty,
     .send_request_finish = s_s3_meta_request_send_request_finish_empty,
     .schedule_prepare_request = s_s3_meta_request_schedule_prepare_request_empty,
-    .prepare_request = s_s3_meta_request_prepare_request_empty,
+    .prepare_request = s_s3_meta_request_prepare_request_async_empty,
     .finished_request = s_s3_meta_request_finished_request_empty,
     .init_signing_date_time = s_s3_meta_request_init_signing_date_time_empty,
     .sign_request = s_s3_meta_request_sign_request_empty,
@@ -1272,6 +1348,7 @@ int aws_s3_tester_send_meta_request_with_options(
     AWS_ZERO_STRUCT(input_stream_buffer);
 
     struct aws_input_stream *input_stream = NULL;
+    struct aws_async_input_stream *async_stream = NULL;
 
     if (meta_request_options.message == NULL) {
         const struct aws_byte_cursor *bucket_name = options->bucket_name;
@@ -1328,12 +1405,6 @@ int aws_s3_tester_send_meta_request_with_options(
                 ASSERT_TRUE(object_size_bytes > client->part_size);
             }
 
-            if (options->put_options.invalid_input_stream) {
-                input_stream = aws_s3_bad_input_stream_new(allocator, object_size_bytes);
-            } else {
-                input_stream = aws_s3_test_input_stream_new(allocator, object_size_bytes);
-            }
-
             struct aws_byte_buf object_path_buffer;
             aws_byte_buf_init(&object_path_buffer, allocator, 128);
 
@@ -1386,28 +1457,33 @@ int aws_s3_tester_send_meta_request_with_options(
 
             struct aws_byte_cursor test_object_path = aws_byte_cursor_from_buf(&object_path_buffer);
             struct aws_byte_cursor host_cur = aws_byte_cursor_from_string(host_name);
-            /* Put together a simple S3 Put Object request. */
-            struct aws_http_message *message = aws_s3_test_put_object_request_new(
-                allocator, &host_cur, test_object_path, g_test_body_content_type, input_stream, options->sse_type);
 
-            if (options->put_options.content_length) {
-                /* make a invalid request */
-                char content_length_buffer[64] = "";
-                snprintf(
-                    content_length_buffer, sizeof(content_length_buffer), "%zu", options->put_options.content_length);
-
-                struct aws_http_headers *headers = aws_http_message_get_headers(message);
-                aws_http_headers_set(
-                    headers, g_content_length_header_name, aws_byte_cursor_from_c_str(content_length_buffer));
+            /* Create "tester" stream with appropriate options */
+            struct aws_async_input_stream_tester_options stream_options = {
+                .base =
+                    {
+                        .autogen_length = object_size_bytes,
+                    },
+            };
+            if (options->put_options.invalid_input_stream) {
+                stream_options.base.fail_on_nth_read = 1;
+                stream_options.base.fail_with_error_code = AWS_IO_STREAM_READ_FAILED;
             }
 
-            if (options->put_options.invalid_request) {
-                /* make a invalid request */
-                aws_http_message_set_request_path(message, aws_byte_cursor_from_c_str("invalid_path"));
+            if (options->put_options.async_input_stream) {
+                stream_options.completion_strategy = AWS_ASYNC_READ_COMPLETES_ON_ANOTHER_THREAD;
+
+                async_stream = aws_async_input_stream_new_tester(allocator, &stream_options);
+                ASSERT_NOT_NULL(async_stream);
+                meta_request_options.send_async_stream = async_stream;
+            } else {
+                input_stream = aws_input_stream_new_tester(allocator, &stream_options.base);
+                ASSERT_NOT_NULL(input_stream);
             }
 
+            /* if uploading via filepath, write input_stream out as tmp file on disk, and then upload that */
             if (options->put_options.file_on_disk) {
-                /* write input_stream to a tmp file on disk */
+                ASSERT_NOT_NULL(input_stream);
                 struct aws_byte_buf filepath_buf;
                 aws_byte_buf_init(&filepath_buf, allocator, 128);
                 struct aws_byte_cursor filepath_prefix = aws_byte_cursor_from_c_str("tmp");
@@ -1438,7 +1514,51 @@ int aws_s3_tester_send_meta_request_with_options(
 
                 /* use filepath instead of input_stream */
                 meta_request_options.send_filepath = aws_byte_cursor_from_string(filepath_str);
-                aws_http_message_set_body_stream(message, NULL);
+                input_stream = aws_input_stream_release(input_stream);
+            }
+
+            /* Put together a simple S3 Put Object request. */
+            struct aws_http_message *message;
+            if (input_stream != NULL) {
+                message = aws_s3_test_put_object_request_new(
+                    allocator, &host_cur, test_object_path, g_test_body_content_type, input_stream, options->sse_type);
+            } else {
+                message = aws_s3_test_put_object_request_new_without_body(
+                    allocator,
+                    &host_cur,
+                    g_test_body_content_type,
+                    test_object_path,
+                    object_size_bytes,
+                    options->sse_type);
+            }
+
+            if (options->put_options.content_length) {
+                /* make a invalid request */
+                char content_length_buffer[64] = "";
+                snprintf(
+                    content_length_buffer, sizeof(content_length_buffer), "%zu", options->put_options.content_length);
+
+                struct aws_http_headers *headers = aws_http_message_get_headers(message);
+                aws_http_headers_set(
+                    headers, g_content_length_header_name, aws_byte_cursor_from_c_str(content_length_buffer));
+            }
+
+            if (options->put_options.skip_content_length) {
+                struct aws_http_headers *headers = aws_http_message_get_headers(message);
+                aws_http_headers_erase(headers, g_content_length_header_name);
+            }
+
+            if (options->put_options.invalid_request) {
+                /* make a invalid request */
+                aws_http_message_set_request_path(message, aws_byte_cursor_from_c_str("invalid_path"));
+            }
+
+            if (options->put_options.content_encoding.ptr != NULL) {
+                struct aws_http_header content_encoding_header = {
+                    .name = g_content_encoding_header_name,
+                    .value = options->put_options.content_encoding,
+                };
+                aws_http_message_add_header(message, content_encoding_header);
             }
 
             meta_request_options.message = message;
@@ -1462,6 +1582,7 @@ int aws_s3_tester_send_meta_request_with_options(
     out_results->headers_callback = options->headers_callback;
     out_results->body_callback = options->body_callback;
     out_results->finish_callback = options->finish_callback;
+    out_results->progress_callback = options->progress_callback;
 
     out_results->algorithm = options->expected_validate_checksum_alg;
 
@@ -1522,6 +1643,8 @@ int aws_s3_tester_send_meta_request_with_options(
     aws_input_stream_release(input_stream);
     input_stream = NULL;
 
+    async_stream = aws_async_input_stream_release(async_stream);
+
     aws_byte_buf_clean_up(&input_stream_buffer);
 
     if (clean_up_local_tester) {
