@@ -57,8 +57,7 @@ static int aead_tls_init(EVP_AEAD_CTX *ctx, const uint8_t *key, size_t key_len,
                          size_t tag_len, enum evp_aead_direction_t dir,
                          const EVP_CIPHER *cipher, const EVP_MD *md,
                          char implicit_iv) {
-  if (tag_len != EVP_AEAD_DEFAULT_TAG_LENGTH &&
-      tag_len != EVP_MD_size(md)) {
+  if (tag_len != EVP_AEAD_DEFAULT_TAG_LENGTH && tag_len != EVP_MD_size(md)) {
     OPENSSL_PUT_ERROR(CIPHER, CIPHER_R_UNSUPPORTED_TAG_SIZE);
     return 0;
   }
@@ -71,14 +70,14 @@ static int aead_tls_init(EVP_AEAD_CTX *ctx, const uint8_t *key, size_t key_len,
   size_t mac_key_len = EVP_MD_size(md);
   size_t enc_key_len = EVP_CIPHER_key_length(cipher);
   assert(mac_key_len + enc_key_len +
-         (implicit_iv ? EVP_CIPHER_iv_length(cipher) : 0) == key_len);
+             (implicit_iv ? EVP_CIPHER_iv_length(cipher) : 0) ==
+         key_len);
 
   AEAD_TLS_CTX *tls_ctx = OPENSSL_malloc(sizeof(AEAD_TLS_CTX));
   if (tls_ctx == NULL) {
-    OPENSSL_PUT_ERROR(CRYPTO, ERR_R_MALLOC_FAILURE);
     return 0;
   }
-  ctx->state.ptr = (void*)tls_ctx;
+  ctx->state.ptr = (void *)tls_ctx;
 
   EVP_CIPHER_CTX_init(&tls_ctx->cipher_ctx);
   HMAC_CTX_init(&tls_ctx->hmac_ctx);
@@ -192,7 +191,8 @@ static int aead_tls_seal_scatter(const EVP_AEAD_CTX *ctx, uint8_t *out,
   // block from encrypting the input and split the result between |out| and
   // |out_tag|. Then feed the rest.
 
-  const size_t early_mac_len = (block_size - (in_len % block_size)) % block_size;
+  const size_t early_mac_len =
+      (block_size - (in_len % block_size)) % block_size;
   if (early_mac_len != 0) {
     assert(len + block_size - early_mac_len == in_len);
     uint8_t buf[EVP_MAX_BLOCK_LENGTH];
@@ -419,9 +419,10 @@ static int aead_aes_256_cbc_sha1_tls_implicit_iv_init(
                        EVP_sha1(), 1);
 }
 
-static int aead_aes_128_cbc_sha256_tls_init(EVP_AEAD_CTX *ctx, const uint8_t *key,
-                                          size_t key_len, size_t tag_len,
-                                          enum evp_aead_direction_t dir) {
+static int aead_aes_128_cbc_sha256_tls_init(EVP_AEAD_CTX *ctx,
+                                            const uint8_t *key, size_t key_len,
+                                            size_t tag_len,
+                                            enum evp_aead_direction_t dir) {
   return aead_tls_init(ctx, key, key_len, tag_len, dir, EVP_aes_128_cbc(),
                        EVP_sha256(), 0);
 }
@@ -469,11 +470,12 @@ static int aead_null_sha1_tls_init(EVP_AEAD_CTX *ctx, const uint8_t *key,
 }
 
 static const EVP_AEAD aead_aes_128_cbc_sha1_tls = {
-    SHA_DIGEST_LENGTH + 16,  // key len (SHA1 + AES128)
-    16,                      // nonce len (IV)
-    16 + SHA_DIGEST_LENGTH,  // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,       // max tag length
-    0,                       // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 16,        // key len (SHA1 + AES128)
+    16,                            // nonce len (IV)
+    16 + SHA_DIGEST_LENGTH,        // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,             // max tag length
+    AEAD_AES_128_CBC_SHA1_TLS_ID,  // evp_aead_id
+    0,                             // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_128_cbc_sha1_tls_init,
@@ -483,14 +485,17 @@ static const EVP_AEAD aead_aes_128_cbc_sha1_tls = {
     NULL,  // open_gather
     NULL,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_aes_128_cbc_sha1_tls_implicit_iv = {
-    SHA_DIGEST_LENGTH + 16 + 16,  // key len (SHA1 + AES128 + IV)
-    0,                            // nonce len
-    16 + SHA_DIGEST_LENGTH,       // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,            // max tag length
-    0,                            // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 16 + 16,               // key len (SHA1 + AES128 + IV)
+    0,                                         // nonce len
+    16 + SHA_DIGEST_LENGTH,                    // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,                         // max tag length
+    AEAD_AES_128_CBC_SHA1_TLS_IMPLICIT_IV_ID,  // evp_aead_id
+    0,                                         // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_128_cbc_sha1_tls_implicit_iv_init,
@@ -500,14 +505,17 @@ static const EVP_AEAD aead_aes_128_cbc_sha1_tls_implicit_iv = {
     NULL,             // open_gather
     aead_tls_get_iv,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_aes_256_cbc_sha1_tls = {
-    SHA_DIGEST_LENGTH + 32,  // key len (SHA1 + AES256)
-    16,                      // nonce len (IV)
-    16 + SHA_DIGEST_LENGTH,  // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,       // max tag length
-    0,                       // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 32,        // key len (SHA1 + AES256)
+    16,                            // nonce len (IV)
+    16 + SHA_DIGEST_LENGTH,        // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,             // max tag length
+    AEAD_AES_256_CBC_SHA1_TLS_ID,  // evp_aead_id
+    0,                             // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_256_cbc_sha1_tls_init,
@@ -517,14 +525,17 @@ static const EVP_AEAD aead_aes_256_cbc_sha1_tls = {
     NULL,  // open_gather
     NULL,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_aes_256_cbc_sha1_tls_implicit_iv = {
-    SHA_DIGEST_LENGTH + 32 + 16,  // key len (SHA1 + AES256 + IV)
-    0,                            // nonce len
-    16 + SHA_DIGEST_LENGTH,       // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,            // max tag length
-    0,                            // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 32 + 16,               // key len (SHA1 + AES256 + IV)
+    0,                                         // nonce len
+    16 + SHA_DIGEST_LENGTH,                    // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,                         // max tag length
+    AEAD_AES_256_CBC_SHA1_TLS_IMPLICIT_IV_ID,  // evp_aead_id
+    0,                                         // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_256_cbc_sha1_tls_implicit_iv_init,
@@ -534,14 +545,17 @@ static const EVP_AEAD aead_aes_256_cbc_sha1_tls_implicit_iv = {
     NULL,             // open_gather
     aead_tls_get_iv,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_aes_128_cbc_sha256_tls = {
-    SHA256_DIGEST_LENGTH + 16,  // key len (SHA256 + AES128)
-    16,                         // nonce len (IV)
-    16 + SHA256_DIGEST_LENGTH,  // overhead (padding + SHA256)
-    SHA256_DIGEST_LENGTH,       // max tag length
-    0,                          // seal_scatter_supports_extra_in
+    SHA256_DIGEST_LENGTH + 16,       // key len (SHA256 + AES128)
+    16,                              // nonce len (IV)
+    16 + SHA256_DIGEST_LENGTH,       // overhead (padding + SHA256)
+    SHA256_DIGEST_LENGTH,            // max tag length
+    AEAD_AES_128_CBC_SHA256_TLS_ID,  // evp_aead_id
+    0,                               // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_128_cbc_sha256_tls_init,
@@ -551,6 +565,8 @@ static const EVP_AEAD aead_aes_128_cbc_sha256_tls = {
     NULL,  // open_gather
     NULL,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_aes_128_cbc_sha256_tls_implicit_iv = {
@@ -558,7 +574,8 @@ static const EVP_AEAD aead_aes_128_cbc_sha256_tls_implicit_iv = {
     0,                               // nonce len
     16 + SHA256_DIGEST_LENGTH,       // overhead (padding + SHA256)
     SHA256_DIGEST_LENGTH,            // max tag length
-    0,                               // seal_scatter_supports_extra_in
+    AEAD_AES_128_CBC_SHA256_TLS_IMPLICIT_IV_ID,  // evp_aead_id
+    0,  // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_aes_128_cbc_sha256_tls_implicit_iv_init,
@@ -568,14 +585,17 @@ static const EVP_AEAD aead_aes_128_cbc_sha256_tls_implicit_iv = {
     NULL,             // open_gather
     aead_tls_get_iv,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_des_ede3_cbc_sha1_tls = {
-    SHA_DIGEST_LENGTH + 24,  // key len (SHA1 + 3DES)
-    8,                       // nonce len (IV)
-    8 + SHA_DIGEST_LENGTH,   // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,       // max tag length
-    0,                       // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 24,         // key len (SHA1 + 3DES)
+    8,                              // nonce len (IV)
+    8 + SHA_DIGEST_LENGTH,          // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,              // max tag length
+    AEAD_DES_EDE3_CBC_SHA1_TLS_ID,  // evp_aead_id
+    0,                              // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_des_ede3_cbc_sha1_tls_init,
@@ -585,14 +605,17 @@ static const EVP_AEAD aead_des_ede3_cbc_sha1_tls = {
     NULL,  // open_gather
     NULL,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_des_ede3_cbc_sha1_tls_implicit_iv = {
-    SHA_DIGEST_LENGTH + 24 + 8,  // key len (SHA1 + 3DES + IV)
-    0,                           // nonce len
-    8 + SHA_DIGEST_LENGTH,       // overhead (padding + SHA1)
-    SHA_DIGEST_LENGTH,           // max tag length
-    0,                           // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH + 24 + 8,                 // key len (SHA1 + 3DES + IV)
+    0,                                          // nonce len
+    8 + SHA_DIGEST_LENGTH,                      // overhead (padding + SHA1)
+    SHA_DIGEST_LENGTH,                          // max tag length
+    AEAD_DES_EDE3_CBC_SHA1_TLS_IMPLICIT_IV_ID,  // evp_aead_id
+    0,  // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_des_ede3_cbc_sha1_tls_implicit_iv_init,
@@ -602,14 +625,17 @@ static const EVP_AEAD aead_des_ede3_cbc_sha1_tls_implicit_iv = {
     NULL,             // open_gather
     aead_tls_get_iv,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 static const EVP_AEAD aead_null_sha1_tls = {
-    SHA_DIGEST_LENGTH,  // key len
-    0,                  // nonce len
-    SHA_DIGEST_LENGTH,  // overhead (SHA1)
-    SHA_DIGEST_LENGTH,  // max tag length
-    0,                  // seal_scatter_supports_extra_in
+    SHA_DIGEST_LENGTH,      // key len
+    0,                      // nonce len
+    SHA_DIGEST_LENGTH,      // overhead (SHA1)
+    SHA_DIGEST_LENGTH,      // max tag length
+    AEAD_NULL_SHA1_TLS_ID,  // evp_aead_id
+    0,                      // seal_scatter_supports_extra_in
 
     NULL,  // init
     aead_null_sha1_tls_init,
@@ -619,6 +645,8 @@ static const EVP_AEAD aead_null_sha1_tls = {
     NULL,  // open_gather
     NULL,  // get_iv
     aead_tls_tag_len,
+    NULL /* serialize_state */,
+    NULL /* deserialize_state */,
 };
 
 const EVP_AEAD *EVP_aead_aes_128_cbc_sha1_tls(void) {
