@@ -51,7 +51,8 @@ void HKDFExpand_verify_service_indicator(const EVP_MD *evp_md);
 void PBKDF2_verify_service_indicator(const EVP_MD *evp_md, size_t password_len,
                                      size_t salt_len, unsigned iterations);
 void SSHKDF_verify_service_indicator(const EVP_MD *evp_md);
-void TLSKDF_verify_service_indicator(const EVP_MD *dgst);
+void TLSKDF_verify_service_indicator(const EVP_MD *dgst, const char *label,
+                                     size_t label_len);
 
 #else
 
@@ -111,7 +112,9 @@ OPENSSL_INLINE void SSHKDF_verify_service_indicator(
     OPENSSL_UNUSED const EVP_MD *evp_md) {}
 
 OPENSSL_INLINE void TLSKDF_verify_service_indicator(
-    OPENSSL_UNUSED const EVP_MD *dgst) {}
+    OPENSSL_UNUSED const EVP_MD *dgst,
+    OPENSSL_UNUSED const char *label,
+    OPENSSL_UNUSED size_t label_len) {}
 
 #endif // AWSLC_FIPS
 
