@@ -28,6 +28,7 @@
 #define OPENSSL_NO_CAST
 #define OPENSSL_NO_CMS
 #define OPENSSL_NO_COMP
+#define OPENSSL_NO_CRYPTO_MDEBUG
 #define OPENSSL_NO_CT
 #define OPENSSL_NO_DANE
 #define OPENSSL_NO_DEPRECATED
@@ -65,6 +66,7 @@
 #define OPENSSL_NO_SSL3_METHOD
 #define OPENSSL_NO_STATIC_ENGINE
 #define OPENSSL_NO_STORE
+#define OPENSSL_NO_TS
 #define OPENSSL_NO_WHIRLPOOL
 
 #endif  // OPENSSL_HEADER_OPENSSLCONF_H
