@@ -13,20 +13,20 @@
  * permissions and limitations under the License.
  */
 
-#include <openssl/evp.h>
-#include "crypto/s2n_openssl_evp.h"
-#include "crypto/s2n_openssl_x509.h"
-
-#include "error/s2n_errno.h"
-#include "crypto/s2n_rsa_pss.h"
 #include "crypto/s2n_pkey.h"
 
+#include <openssl/evp.h>
+
+#include "crypto/s2n_openssl_evp.h"
+#include "crypto/s2n_openssl_x509.h"
+#include "crypto/s2n_rsa_pss.h"
+#include "error/s2n_errno.h"
 #include "utils/s2n_result.h"
 #include "utils/s2n_safety.h"
 
 #define S2N_MAX_ALLOWED_CERT_TRAILING_BYTES 3
 
-int s2n_pkey_zero_init(struct s2n_pkey *pkey) 
+int s2n_pkey_zero_init(struct s2n_pkey *pkey)
 {
     pkey->pkey = NULL;
     pkey->size = NULL;
@@ -42,7 +42,7 @@ int s2n_pkey_zero_init(struct s2n_pkey *pkey)
 
 int s2n_pkey_setup_for_type(struct s2n_pkey *pkey, s2n_pkey_type pkey_type)
 {
-    switch(pkey_type) {
+    switch (pkey_type) {
         case S2N_PKEY_TYPE_RSA:
             return s2n_rsa_pkey_init(pkey);
         case S2N_PKEY_TYPE_ECDSA:
@@ -79,7 +79,7 @@ int s2n_pkey_sign(const struct s2n_pkey *pkey, s2n_signature_algorithm sig_alg,
         struct s2n_hash_state *digest, struct s2n_blob *signature)
 {
     POSIX_ENSURE_REF(pkey->sign);
-    
+
     return pkey->sign(pkey, sig_alg, digest, signature);
 }
 
@@ -129,15 +129,30 @@ int s2n_pkey_free(struct s2n_pkey *key)
     return S2N_SUCCESS;
 }
 
-int s2n_asn1der_to_private_key(struct s2n_pkey *priv_key, struct s2n_blob *asn1der)
+int s2n_asn1der_to_private_key(struct s2n_pkey *priv_key, struct s2n_blob *asn1der, int type_hint)
 {
-    uint8_t *key_to_parse = asn1der->data;
+    const unsigned char *key_to_parse = asn1der->data;
 
-    /* Detect key type */
-    DEFER_CLEANUP(EVP_PKEY *evp_private_key = d2i_AutoPrivateKey(NULL, (const unsigned char **)(void *)&key_to_parse, asn1der->size),
+    /* We use "d2i_AutoPrivateKey" instead of "PEM_read_bio_PrivateKey" because
+     * s2n-tls prefers to perform its own custom PEM parsing. Historically,
+     * openssl's PEM parsing tended to ignore invalid certificates rather than
+     * error on them. We prefer to fail early rather than continue without
+     * the full and correct chain intended by the application.
+     */
+    DEFER_CLEANUP(EVP_PKEY *evp_private_key = d2i_AutoPrivateKey(NULL, &key_to_parse, asn1der->size),
             EVP_PKEY_free_pointer);
-    S2N_ERROR_IF(evp_private_key == NULL, S2N_ERR_DECODE_PRIVATE_KEY);
-    
+
+    /* We have found cases where d2i_AutoPrivateKey fails to detect the type of
+     * the key. For example, openssl fails to identify an EC key without the
+     * optional publicKey field.
+     *
+     * If d2i_AutoPrivateKey fails, try once more with the type we parsed from the PEM.
+     */
+    if (evp_private_key == NULL) {
+        evp_private_key = d2i_PrivateKey(type_hint, NULL, &key_to_parse, asn1der->size);
+    }
+    POSIX_ENSURE(evp_private_key, S2N_ERR_DECODE_PRIVATE_KEY);
+
     /* If key parsing is successful, d2i_AutoPrivateKey increments *key_to_parse to the byte following the parsed data */
     uint32_t parsed_len = key_to_parse - asn1der->data;
     if (parsed_len != asn1der->size) {
@@ -146,34 +161,34 @@ int s2n_asn1der_to_private_key(struct s2n_pkey *priv_key, struct s2n_blob *asn1d
 
     /* Initialize s2n_pkey according to key type */
     int type = EVP_PKEY_base_id(evp_private_key);
-    
+
     int ret;
     switch (type) {
-    case EVP_PKEY_RSA:
-        ret = s2n_rsa_pkey_init(priv_key);
-        if (ret != 0) {
+        case EVP_PKEY_RSA:
+            ret = s2n_rsa_pkey_init(priv_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_rsa_private_key(&priv_key->key.rsa_key, evp_private_key);
             break;
-        }
-        ret = s2n_evp_pkey_to_rsa_private_key(&priv_key->key.rsa_key, evp_private_key);
-        break;
-    case EVP_PKEY_RSA_PSS:
-        ret = s2n_rsa_pss_pkey_init(priv_key);
-        if (ret != 0) {
+        case EVP_PKEY_RSA_PSS:
+            ret = s2n_rsa_pss_pkey_init(priv_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_rsa_pss_private_key(&priv_key->key.rsa_key, evp_private_key);
             break;
-        }
-        ret = s2n_evp_pkey_to_rsa_pss_private_key(&priv_key->key.rsa_key, evp_private_key);
-        break;
-    case EVP_PKEY_EC:
-        ret = s2n_ecdsa_pkey_init(priv_key);
-        if (ret != 0) {
+        case EVP_PKEY_EC:
+            ret = s2n_ecdsa_pkey_init(priv_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_ecdsa_private_key(&priv_key->key.ecdsa_key, evp_private_key);
             break;
-        }
-        ret = s2n_evp_pkey_to_ecdsa_private_key(&priv_key->key.ecdsa_key, evp_private_key);
-        break;
-    default:
-        POSIX_BAIL(S2N_ERR_DECODE_PRIVATE_KEY);
+        default:
+            POSIX_BAIL(S2N_ERR_DECODE_PRIVATE_KEY);
     }
-    
+
     priv_key->pkey = evp_private_key;
     /* Reset to avoid DEFER_CLEANUP freeing our key */
     evp_private_key = NULL;
@@ -186,7 +201,7 @@ int s2n_asn1der_to_public_key_and_type(struct s2n_pkey *pub_key, s2n_pkey_type *
     uint8_t *cert_to_parse = asn1der->data;
     DEFER_CLEANUP(X509 *cert = NULL, X509_free_pointer);
 
-    cert = d2i_X509(NULL, (const unsigned char **)(void *)&cert_to_parse, asn1der->size);
+    cert = d2i_X509(NULL, (const unsigned char **) (void *) &cert_to_parse, asn1der->size);
     S2N_ERROR_IF(cert == NULL, S2N_ERR_DECODE_CERTIFICATE);
 
     /* If cert parsing is successful, d2i_X509 increments *cert_to_parse to the byte following the parsed data */
@@ -205,32 +220,32 @@ int s2n_asn1der_to_public_key_and_type(struct s2n_pkey *pub_key, s2n_pkey_type *
 
     int ret;
     switch (type) {
-    case EVP_PKEY_RSA:
-        ret = s2n_rsa_pkey_init(pub_key);
-        if (ret != 0) {
+        case EVP_PKEY_RSA:
+            ret = s2n_rsa_pkey_init(pub_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_rsa_public_key(&pub_key->key.rsa_key, evp_public_key);
+            *pkey_type_out = S2N_PKEY_TYPE_RSA;
             break;
-        }
-        ret = s2n_evp_pkey_to_rsa_public_key(&pub_key->key.rsa_key, evp_public_key);
-        *pkey_type_out = S2N_PKEY_TYPE_RSA;
-        break;
-    case EVP_PKEY_RSA_PSS:
-        ret = s2n_rsa_pss_pkey_init(pub_key);
-        if (ret != 0) {
+        case EVP_PKEY_RSA_PSS:
+            ret = s2n_rsa_pss_pkey_init(pub_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_rsa_pss_public_key(&pub_key->key.rsa_key, evp_public_key);
+            *pkey_type_out = S2N_PKEY_TYPE_RSA_PSS;
             break;
-        }
-        ret = s2n_evp_pkey_to_rsa_pss_public_key(&pub_key->key.rsa_key, evp_public_key);
-        *pkey_type_out = S2N_PKEY_TYPE_RSA_PSS;
-        break;
-    case EVP_PKEY_EC:
-        ret = s2n_ecdsa_pkey_init(pub_key);
-        if (ret != 0) {
+        case EVP_PKEY_EC:
+            ret = s2n_ecdsa_pkey_init(pub_key);
+            if (ret != 0) {
+                break;
+            }
+            ret = s2n_evp_pkey_to_ecdsa_public_key(&pub_key->key.ecdsa_key, evp_public_key);
+            *pkey_type_out = S2N_PKEY_TYPE_ECDSA;
             break;
-        }
-        ret = s2n_evp_pkey_to_ecdsa_public_key(&pub_key->key.ecdsa_key, evp_public_key);
-        *pkey_type_out = S2N_PKEY_TYPE_ECDSA;
-        break;
-    default:
-        POSIX_BAIL(S2N_ERR_DECODE_CERTIFICATE);
+        default:
+            POSIX_BAIL(S2N_ERR_DECODE_CERTIFICATE);
     }
 
     pub_key->pkey = evp_public_key;
@@ -239,4 +254,3 @@ int s2n_asn1der_to_public_key_and_type(struct s2n_pkey *pub_key, s2n_pkey_type *
 
     return ret;
 }
-
