@@ -531,20 +531,12 @@ void *aws_py_get_binding(PyObject *obj, const char *capsule_name, const char *cl
 
     PyObject *py_binding = PyObject_GetAttrString(obj, "_binding"); /* new reference */
     if (!py_binding) {
-        return PyErr_Format(
-            PyExc_TypeError,
-            "Expected valid '%s', received '%s' (no '_binding' attribute)",
-            class_name,
-            Py_TYPE(obj)->tp_name);
+        return PyErr_Format(PyExc_TypeError, "Expected valid '%s' (no '_binding' attribute)", class_name);
     }
 
     void *binding = NULL;
     if (!PyCapsule_CheckExact(py_binding)) {
-        PyErr_Format(
-            PyExc_TypeError,
-            "Expected valid '%s', received '%s' ('_binding' attribute is not a capsule)",
-            class_name,
-            Py_TYPE(obj)->tp_name);
+        PyErr_Format(PyExc_TypeError, "Expected valid '%s' ('_binding' attribute is not a capsule)", class_name);
         goto done;
     }
 
@@ -552,9 +544,8 @@ void *aws_py_get_binding(PyObject *obj, const char *capsule_name, const char *cl
     if (!binding) {
         PyErr_Format(
             PyExc_TypeError,
-            "Expected valid '%s', received '%s' ('_binding' attribute does not contain '%s')",
+            "Expected valid '%s' ('_binding' attribute does not contain '%s')",
             class_name,
-            Py_TYPE(obj)->tp_name,
             capsule_name);
         goto done;
     }
@@ -706,6 +697,7 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(mqtt_client_connection_unsubscribe, METH_VARARGS),
     AWS_PY_METHOD_DEF(mqtt_client_connection_disconnect, METH_VARARGS),
     AWS_PY_METHOD_DEF(mqtt_ws_handshake_transform_complete, METH_VARARGS),
+    AWS_PY_METHOD_DEF(mqtt_client_connection_get_stats, METH_VARARGS),
 
     /* MQTT5 Client */
     AWS_PY_METHOD_DEF(mqtt5_client_new, METH_VARARGS),
@@ -769,6 +761,7 @@ static PyMethodDef s_module_methods[] = {
     AWS_PY_METHOD_DEF(credentials_provider_new_chain, METH_VARARGS),
     AWS_PY_METHOD_DEF(credentials_provider_new_delegate, METH_VARARGS),
     AWS_PY_METHOD_DEF(credentials_provider_new_cognito, METH_VARARGS),
+    AWS_PY_METHOD_DEF(credentials_provider_new_x509, METH_VARARGS),
     AWS_PY_METHOD_DEF(signing_config_new, METH_VARARGS),
     AWS_PY_METHOD_DEF(signing_config_get_algorithm, METH_VARARGS),
     AWS_PY_METHOD_DEF(signing_config_get_signature_type, METH_VARARGS),
